import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;

class BufferedWriterDemo {
	public static void main(String[] args) throws IOException {
		final int ANZAHL = 5_000_000;

		try (FileWriter fw = new FileWriter("test1.txt")) {
			long time = System.currentTimeMillis();
			for (int i = 1; i <= ANZAHL; i++)
				fw.write("Zeile " + i + System.lineSeparator());
			System.out.println("Benötigte Zeit beim Warmlaufen:            "
					+(System.currentTimeMillis()-time));
		}

		try (FileWriter fw = new FileWriter("test2.txt")) {
			long time = System.currentTimeMillis();
			for (int i = 1; i <= ANZAHL; i++)
				fw.write("Zeile " + i + System.lineSeparator());
			System.out.println("Benötigte Zeit ohne BufferedWriter:        "
					+(System.currentTimeMillis()-time));
		}

		try (BufferedWriter bw =  new BufferedWriter(new FileWriter("test3.txt"))) {
			long time = System.currentTimeMillis();
			for (int i = 1; i <= ANZAHL; i++)
				bw.write("Zeile " + i + System.lineSeparator());
			System.out.println("Benötigte Zeit mit BufferedWriter:         "
					+(System.currentTimeMillis()-time));
		}

		try (BufferedWriter bw =  Files.newBufferedWriter(Paths.get("test4.txt"), Charset.defaultCharset())) {
			long time = System.currentTimeMillis();
			for (int i = 1; i <= ANZAHL; i++)
				bw.write("Zeile " + i + System.lineSeparator());
			System.out.println("Benötigte Zeit mit BufferedWriter (NIO2.): "
					+(System.currentTimeMillis()-time));
		}
		
		Files.delete(Paths.get("test1.txt"));
		Files.delete(Paths.get("test2.txt"));
		Files.delete(Paths.get("test3.txt"));
		Files.delete(Paths.get("test4.txt"));
	}
}