import java.math.BigDecimal;
import java.util.concurrent.*;


public class FacForkJoin {
	static private ForkJoinPool pool = new ForkJoinPool();

	private FacForkJoin() {}
	
	static public BigDecimal factorial(int argument, int schwelle) {
		FacTask task = new FacTask(1, argument, schwelle);
		pool.invoke(task);
		return task.join();
	}
	
	static public BigDecimal factorial(int argument) {
		return factorial(argument, Runtime.getRuntime().availableProcessors()*10);
	}
	
	@SuppressWarnings("serial")
	private static class FacTask extends RecursiveTask<BigDecimal> {
		private int start, ende;
		private int schwelle;
		
		public FacTask(int start, int ende, int schwelle) {
			this.start = start;
			this.ende = ende; 
			this.schwelle = schwelle;
		}
		
		private BigDecimal product(int start, int ende) {
			BigDecimal fac = new BigDecimal(start);
			for (int i = start+1; i <= ende; i++)
				fac = fac.multiply(new BigDecimal(i));
			return fac;
		}
		
		@Override
		protected BigDecimal compute() {
			BigDecimal fac;
			int umfang = ende - start; 
			if (umfang <= schwelle)
				fac = product(start, ende);
			else {
				int haelfte = umfang/2;
				FacTask task1 = new FacTask(start, start+haelfte, schwelle);
				FacTask task2 = new FacTask(start+haelfte+1, ende, schwelle);
				task2.fork();
				BigDecimal erg1 = task1.compute();
				BigDecimal erg2 = task2.join();
				fac = erg1.multiply(erg2);
			}
			return fac;
		}
	}
}