import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class SwingWorkerDemo  {
	private JFrame frame;
	private final static String labelPrefix = "Zählerstand: ";
	private final JLabel lblCounter, lblRandom;
	private final JPanel panCenter;
	private final JButton cbCount, cbWorker;
	private int numClicks = 0;
	private final long anz = 100_000_000;
	
	SwingWorkerDemo() {
		frame = new JFrame("SwingWorker");
		Container cont = frame.getContentPane();

		cbCount = new JButton("Zählerstand erhöhen");
		frame.getRootPane().setDefaultButton(cbCount);
		cbCount.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				numClicks++;
				lblCounter.setText(labelPrefix + numClicks);
			}
		});

		cbWorker = new JButton("Mittel von "+anz+" Zufallszahlen");
		cbWorker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lblRandom.setText("Berechnung läuft ...");
				(new RandomNumberCruncher()).execute();
			}
		});
		
		lblCounter = new JLabel(labelPrefix + "0    ");
		Font f = new Font("SansSerif", Font.BOLD, 16);
		lblCounter.setFont(f);

		lblRandom = new JLabel();
		lblRandom.setFont(f);
		
		panCenter = new JPanel();
		panCenter.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		panCenter.setLayout(new GridLayout(0, 1));
		panCenter.add(cbCount);
		panCenter.add(lblCounter);
		panCenter.add(lblRandom);
		panCenter.add(cbWorker);
		cont.add(panCenter, BorderLayout.CENTER);

		frame.setMinimumSize(new Dimension(500, 200));		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}

	private class RandomNumberCruncher extends SwingWorker<Double, Void> {
		public Double doInBackground() {
			double d = 0.0;
			for (int i = 0; i < anz; i++)
				d += Math.random();
			return new Double(d/anz);
		}
		
		public void done() {
			try {
				lblRandom.setText("Mittelwert der Zufallszahlen: " + get());
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "Berechnung gescheitert");
			}
		}
	}

	public static void main(String args[]) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				new SwingWorkerDemo();
			}
		});
	}
}

