import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;

import java.io.*;

class Editor {
	private final JFrame frame;
	private final JTextPane text;
	private File file;
	
	public Editor() {
		frame= new JFrame("Editor - mit Hintergrundfarbe");

		text = new JTextPane();
		text.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 14));
		frame.getContentPane().add(new JScrollPane(text), BorderLayout.CENTER);
		
		final JMenuBar menuBar = new JMenuBar();
		frame.setJMenuBar(menuBar);
		
		final JMenu fileMenu = new JMenu("Datei");
		fileMenu.setMnemonic(KeyEvent.VK_D);
		menuBar.add(fileMenu);
		
		final JMenuItem openItem = new JMenuItem("Öffnen");
		openItem.setMnemonic(KeyEvent.VK_F);
		openItem.setIcon(new ImageIcon("open.gif"));
		fileMenu.add(openItem);
		openItem.addActionListener(new	ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getFileName();
					if (file != null) readText();
				}
			});

		// Trennlinie zwischen Menüeinträgen
		fileMenu.addSeparator();
		
		final JMenuItem exitItem = new JMenuItem("Beenden");
		exitItem.setMnemonic(KeyEvent.VK_B);
		exitItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		fileMenu.add(exitItem);

		final JMenu toolsMenu = new JMenu("Extras");
		toolsMenu.setMnemonic(KeyEvent.VK_E);
		final JMenu fontMenu = new JMenu("Schriftart");
		fontMenu.setIcon(new ImageIcon("font.gif"));		
		fontMenu.setMnemonic(KeyEvent.VK_S);
		toolsMenu.add(fontMenu);

		final JMenuItem timesItem = new JRadioButtonMenuItem("Times-Roman");
		timesItem.setMnemonic(KeyEvent.VK_T);
		timesItem.addActionListener(new	ActionListener() {
				public void actionPerformed(ActionEvent e) {
					text.setFont(new Font(Font.SERIF, Font.PLAIN, 14));
					timesItem.setSelected(true);					
				}
			});
		fontMenu.add(timesItem);

		final JMenuItem arialItem = new JRadioButtonMenuItem("Arial");
		arialItem.setMnemonic(KeyEvent.VK_A);
		arialItem.setSelected(true);
		arialItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					text.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 14));
					arialItem.setSelected(true);
				}
			});
		fontMenu.add(arialItem);
		
		ButtonGroup bgFonts = new ButtonGroup();
		bgFonts.add(timesItem);
		bgFonts.add(arialItem);

		menuBar.add(toolsMenu);

		final JMenuItem colorItem = new JMenuItem("Hintergrundfarbe");
		colorItem.setMnemonic(KeyEvent.VK_H);
		colorItem.setIcon(new ImageIcon("color.gif"));
		toolsMenu.add(colorItem);
		colorItem.addActionListener(new	ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Color tempColor = 
						JColorChooser.showDialog(text,
								"Wählen Sie eine Hintergrundfarbe.", text.getBackground());
					if (tempColor != null)
						text.setBackground(tempColor);
				}
			});
		menuBar.add(toolsMenu);
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(300, 200);
		frame.setVisible(true);
		text.requestFocusInWindow();	
	}

	private void getFileName() {
		FileNameExtensionFilter filter = 
			new FileNameExtensionFilter("Textdatei (*.txt)", "txt");
		JFileChooser fc = new JFileChooser();
		fc.addChoosableFileFilter(filter);
		if (fc.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION)
			file = fc.getSelectedFile();
	}

	private void readText() {
		try (BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file)) ) {
			text.read(bis, null);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(frame,
				"Fehler beim Lesen:\n"+e, "Fehler",
				 JOptionPane.ERROR_MESSAGE);
		}
	}

	public static void main(String[] arg) {
		new Editor();
	}
}