import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.filechooser.*;

import java.io.*;

enum FileDialogType {OPEN, SAVE}

class Editor {
	private final JFrame frame;
	private final JTextPane text;
	private File file;
	private final JFileChooser fc;
	private final JMenuItem[] lafItems;
	private final UIManager.LookAndFeelInfo[] lafs;
	private final static String titel = "Editor - mit Look & Feel";
	
	public Editor() {
		frame = new JFrame(titel);

		text = new JTextPane();
		text.setDragEnabled(true);
		text.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 14));
		frame.getContentPane().add(new JScrollPane(text), BorderLayout.CENTER);

		// JFileChooser vorbereiten
		final FileNameExtensionFilter filter = 
			new FileNameExtensionFilter("Textdatei (*.txt)", "txt");
		fc = new JFileChooser();
		fc.setFileFilter(filter);
		
		final JMenuBar menuBar = new JMenuBar();
		frame.setJMenuBar(menuBar);
		
		final JMenu fileMenu = new JMenu("Datei");
		fileMenu.setMnemonic(KeyEvent.VK_D);
		menuBar.add(fileMenu);
		
		final JMenuItem openItem = new JMenuItem("Öffnen");
		openItem.setMnemonic(KeyEvent.VK_F);
		openItem.setIcon(new ImageIcon("open.gif"));
		fileMenu.add(openItem);
		openItem.addActionListener(new	ActionListener() {
				public void actionPerformed(ActionEvent e) {
					ask4File(FileDialogType.OPEN);
					if (file != null)
						readText();
				}
			});

		final JMenuItem saveItem = new JMenuItem("Sichern");
		saveItem.setMnemonic(KeyEvent.VK_S);
		saveItem.setIcon(new ImageIcon("save.gif"));		
		fileMenu.add(saveItem);
		saveItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (file == null)
						ask4File(FileDialogType.SAVE);
					writeText();
				}
			});

		final JMenuItem saveAsItem = new JMenuItem("Sichern unter");
		saveAsItem.setMnemonic(KeyEvent.VK_U);
		saveAsItem.setIcon(new ImageIcon("saveAs.gif"));		
		fileMenu.add(saveAsItem);
		saveAsItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					ask4File(FileDialogType.SAVE);
					if (file != null) {
						writeText();
					}
				}
			});
		
		// Trennlinie zwischen Menüeinträgen
		fileMenu.addSeparator();
		
		final JMenuItem exitItem = new JMenuItem("Beenden");
		exitItem.setMnemonic(KeyEvent.VK_B);
		exitItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		fileMenu.add(exitItem);

		final JMenu toolsMenu = new JMenu("Extras");
		toolsMenu.setMnemonic(KeyEvent.VK_E);
		final JMenu fontMenu = new JMenu("Schriftart");
		fontMenu.setMnemonic(KeyEvent.VK_S);
		fontMenu.setIcon(new ImageIcon("font.gif"));
		toolsMenu.add(fontMenu);

		final JMenuItem timesItem = new JRadioButtonMenuItem("Times-Roman");
		timesItem.setMnemonic(KeyEvent.VK_T);
		timesItem.addActionListener(new	ActionListener() {
				public void actionPerformed(ActionEvent e) {
					text.setFont(new Font(Font.SERIF, Font.PLAIN, 14));
					timesItem.setSelected(true);
				}
			});
		fontMenu.add(timesItem);

		final JMenuItem arialItem = new JRadioButtonMenuItem("Arial");
		arialItem.setMnemonic(KeyEvent.VK_A);
		arialItem.setSelected(true);
		arialItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					text.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 14));
					arialItem.setSelected(true);
				}
			});
		fontMenu.add(arialItem);

		final ButtonGroup bgFonts = new ButtonGroup();
		bgFonts.add(timesItem);
		bgFonts.add(arialItem);
		
		final JMenu lafMenu = new JMenu("Look & Feel");
		lafs = UIManager.getInstalledLookAndFeels();
		lafItems = new JMenuItem[lafs.length];
		final ButtonGroup bgLaf = new ButtonGroup();
		final UIActionHandler uah = new UIActionHandler(); 
		for (int i = 0; i< lafs.length; i++) {
			lafItems[i] = new JMenuItem(lafs[i].getName());
			bgLaf.add(lafItems[i]);
			lafItems[i].addActionListener(uah);
			lafMenu.add(lafItems[i]);
		}
		toolsMenu.add(lafMenu);

		menuBar.add(toolsMenu);

		final JMenuItem colorItem = new JMenuItem("Hintergrundfarbe");
		colorItem.setMnemonic(KeyEvent.VK_H);
		colorItem.setIcon(new ImageIcon("color.gif"));
		toolsMenu.add(colorItem);
		colorItem.addActionListener(new	ActionListener() {
				public void actionPerformed(ActionEvent e) {
					final Color tempColor = 
						JColorChooser.showDialog(text,
								"Wählen Sie eine Hintergrundfarbe.", text.getBackground());
					if (tempColor != null)
						text.setBackground(tempColor);
				}
			});
		menuBar.add(toolsMenu);
		
		// Symbolleiste erstellen
		final JToolBar toolBar = new JToolBar("Editor");
		frame.getContentPane().add(toolBar, BorderLayout.NORTH);
		menuBar.setBorder(BorderFactory.createEtchedBorder());
		// toolBar.setFloatable(false);
		
		final JButton openButton = new JButton(new ImageIcon("open.gif"));
		openButton.addActionListener(openItem.getActionListeners()[0]);
		toolBar.add(openButton);

		toolBar.addSeparator();

		final JButton saveButton = new JButton(new ImageIcon("save.gif"));
		saveButton.addActionListener(saveItem.getActionListeners()[0]);
		toolBar.add(saveButton);
		
		toolBar.addSeparator();

		final JButton colorButton = new JButton(new ImageIcon("color.gif"));
		colorButton.addActionListener(colorItem.getActionListeners()[0]);
		toolBar.add(colorButton);
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(300, 200);
		frame.setVisible(true);
		text.requestFocusInWindow();		
	}

	private void ask4File(FileDialogType type) {
		int returnCode;
		if (type == FileDialogType.OPEN)
			returnCode = fc.showOpenDialog(frame);
		else
			returnCode = fc.showSaveDialog(frame);
		if (returnCode == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		}
	}
	
	private void readText() {
		try (BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file)) ) {
			text.read(bis, null);
		} catch (final Exception e) {
			JOptionPane.showMessageDialog(frame,
				"Fehler beim Lesen:\n"+e, "Fehler",
				 JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private void writeText() {
		try (PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file))) ) {
			text.write(pw);
		} catch (final Exception e) {
			JOptionPane.showMessageDialog(frame,
				"Fehler beim Schreiben:\n"+e, "Fehler",
				 JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private class UIActionHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			for (int i = 0; i < lafs.length; i++) {
				if (e.getSource() == lafItems[i])
					try {
						UIManager.setLookAndFeel(lafs[i].getClassName());
						SwingUtilities.updateComponentTreeUI(Editor.this.frame);
						lafItems[i].setSelected(true);
						break;
					} catch (final Exception ex) {
						JOptionPane.showMessageDialog(text, "Fehler beim UI-Management", titel, JOptionPane.ERROR_MESSAGE);
					}
			}
		}
	}
	
	public static void main(String[] arg) {
		new Editor();
	}
}