import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.NumberFormat;
import java.text.ParsePosition;

enum Richtung {DM2EURO, EURO2DM};

class EuroKonverter {
	private JLabel lblEingabe, lblAusgabe;
	private JTextField tfEingabe, tfAusgabe;	
	private JRadioButton euro2dm, dm2euro;
	private Richtung richtung = Richtung.DM2EURO;
	// Wird zur Erkennung überbreiter Ausgaben benötigt:
	private FontMetrics fm;

    EuroKonverter() {
    	JFrame frame = new JFrame("Euro-Konverter");
        final Container cont = frame.getContentPane();
        cont.setLayout(new GridLayout(3, 0));
        
		// Dem Unicode-Zeichen \u27a0 entspricht ein horizontaler Pfeil. 
		dm2euro = new JRadioButton("DM "+'\u27a0'+" Euro", true);
   		dm2euro.setFocusable(false);

		euro2dm = new JRadioButton("Euro "+'\u27a0'+" DM", false);
   		euro2dm.setFocusable(false);
				
   		ButtonGroup rbGroup = new ButtonGroup();
		rbGroup.add(euro2dm);
		rbGroup.add(dm2euro);
		
		final RadioHandler rbHandler = new RadioHandler();
		euro2dm.addItemListener(rbHandler);
		dm2euro.addItemListener(rbHandler);

		JButton cbBeenden = new JButton("Ende");
   		cbBeenden.setFocusable(false);
        cbBeenden.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });

		tfEingabe = new JTextField("", 9);

		tfEingabe.setHorizontalAlignment(JTextField.RIGHT);
		lblEingabe = new JLabel("DM");
		
		tfAusgabe = new JTextField("", 9);
   		tfAusgabe.setEditable(false);
   		tfAusgabe.setFocusable(false);
   		tfAusgabe.setHorizontalAlignment(JTextField.RIGHT);
		lblAusgabe = new JLabel("Euro");
		
		// Konvertiert Zeichenfolge in Zahl und umgekehrt; unterstützt das lokal-übliche Zahlenformat
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		
		JButton cbKonvertieren = new JButton("  Konvertieren  ");
        cbKonvertieren.addActionListener(new ActionListener() {
        	ParsePosition pp = new ParsePosition(0);
        	public void actionPerformed(ActionEvent e) {
          		final String eingabe = tfEingabe.getText();
				String ausgabe;
          		if (eingabe.length() == 0) {
          			tfEingabe.requestFocus();
          			return;
          		}
          		double factor = 1.95583;
          		if (richtung == Richtung.EURO2DM) {
          			factor = 1.95583;
          		}
          		else {
          			factor = 1.0/1.95583;
          		}
      			try {
      				// parse() akzeptiert z.B. auch "1komma3". Mit Hilfe des ParsePosition-Objekts wird erkannt, wenn die Interpretation vor dem Ende der Zeichenfolge stoppt.
      				pp.setIndex(0);
      				final Number n = nf.parse(eingabe, pp);
      				if (pp.getIndex() != eingabe.length())
      					throw new NumberFormatException("Die numerische Interpretation der Eingabe wurde abgebrochen");
      				double wert = n.doubleValue();
	      			wert *= factor;
	      			ausgabe = nf.format(wert);
	      			if (fm.stringWidth(ausgabe) <= tfAusgabe.getWidth())
						tfAusgabe.setText(ausgabe);
					else
						tfAusgabe.setText("Ergebnis zu breit!");
      			} catch (final Exception ex) {
      				JOptionPane.showMessageDialog(frame, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE);
      			}
//				cbKonvertieren.transferFocus();
				tfEingabe.requestFocus();
				tfEingabe.selectAll();
            }
        });
        frame.getRootPane().setDefaultButton(cbKonvertieren);
        
        JPanel panLinksOben = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panLinksOben.add(dm2euro);
		cont.add(panLinksOben);

		JPanel panRechtsOben = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panRechtsOben.add(tfEingabe);
		panRechtsOben.add(lblEingabe);
		cont.add(panRechtsOben);

		JPanel panLinksMitte = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panLinksMitte.add(euro2dm);
		cont.add(panLinksMitte);

		JPanel panRechtsMitte = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panRechtsMitte.add(tfAusgabe);
		panRechtsMitte.add(lblAusgabe);
		cont.add(panRechtsMitte);

		JPanel panLinksUnten = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panLinksUnten.add(cbBeenden);
		cont.add(panLinksUnten);

		JPanel panRechtsUnten = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panRechtsUnten.add(cbKonvertieren);
		cont.add(panRechtsUnten);
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setResizable(false);
		frame.pack();
		// Wird zur Erkennung überbreiter Ausgaben benötigt, setzt pack() voraus
		fm = frame.getFontMetrics(frame.getFont());
		tfEingabe.requestFocusInWindow();
		frame.setVisible(true);
    }

    public static void main(String[] args) {
        new EuroKonverter();
    }

	class RadioHandler implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			tfEingabe.setText("");
			tfAusgabe.setText("");
			if (e.getSource() == euro2dm) {
				lblEingabe.setText("Euro");
				lblAusgabe.setText("DM");
				richtung = Richtung.EURO2DM;
			} else if (e.getSource() == dm2euro) {
				lblEingabe.setText("DM");
				lblAusgabe.setText("Euro");
				richtung = Richtung.DM2EURO;
			}
		}
	}
}
