import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class MPC {
	private final static String TITLE = "Multi Purpose Counter";
	private int numClicks = 0;
	private int iconInd = 0;

	MPC() {
		// Über den JFrame-Konstruktor erhält das Rahmenfenster einen Titel.
		final JFrame frame = new JFrame("Multi Purpose Counter");
		
		// cont ist eine bequeme Referenz auf die Inhaltsschicht des JFrame-Fensters
		final Container cont = frame.getContentPane();
		
		// JLabel-Komponente mit Text initialisieren
		final String lblPrefix = "Zählerstand: ";
		final JLabel lblText = new JLabel(lblPrefix + "0");
		lblText.setToolTipText("Label mit Text");
		
		// Icons erzeugen
		final ImageIcon[] icons = new ImageIcon[3];
		icons[0] = new ImageIcon("duke.gif");
		icons[1] = new ImageIcon("fight.gif");
		icons[2] = new ImageIcon("snooze.gif");
		
		// JLabel mit Icon initialisieren
		final JLabel lblIcon = new JLabel(icons[0]);
		lblIcon.setToolTipText("Label mit Icon");

		// JButton-Komponente initialisieren
		final JButton cbCount = new JButton("Zählerstand erhöhen");
		cbCount.setMnemonic(KeyEvent.VK_A);
		
		// Tatsächliche Klassen von rootPane, contentPane und glassPane ermitteln
//		System.out.println(getRootPane().getContentPane().getClass().getName());
//		System.out.println(cont.getClass().getName());
//		System.out.println(getRootPane().getGlassPane().getClass().getName());
		
		// Der Schalter kann als Default Button auch mit Enter ausgelöst werden.		
		frame.getRootPane().setDefaultButton(cbCount);

		// Quick-Info zum Befehlsschalter		
		cbCount.setToolTipText("Befehlsschalter");

		// JPanel-Komponente erzeugen.
		// Dieser "immediate container" dient zur Positionierung von Komponenten.
		final JPanel panLeft = new JPanel();
		
		// Der Container wird "frei gestellt".
		panLeft.setBorder(BorderFactory.createEmptyBorder(30, 30, 30, 30));
		
		// Layout-Manager für die JPanel-Komponente erzeugen
		// Hier wird dafür gesorgt, dass alle Komponenten in einer Spalte stehen.
		panLeft.setLayout(new GridLayout(0, 1));
		
		// Der JButton und das Text-JLabel werden auf die JPanel-Komponente gesetzt.
		panLeft.add(cbCount);
		panLeft.add(lblText);
		
		// Der JPanel-Container wird auf den JFrame gesetzt (Position: Mitte).
		cont.add(panLeft, BorderLayout.CENTER);

		// Das Icon-Label wird auf den JFrame gesetzt (Position: Osten, also rechter Rand).
		cont.add(lblIcon, BorderLayout.EAST);
		
		// Für den JButton wird ein ActionListener registriert.
		// Dazu wird eine anonyme Klasse an Ort und Stelle definiert.
		cbCount.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				numClicks++;
				lblText.setText(lblPrefix + numClicks);
				if (iconInd < icons.length-1)
					iconInd++;
				else
					iconInd = 0;
				lblIcon.setIcon(icons[iconInd]);
			}
		});
		
		// Seit Java 8 lässt sich ein Lambda-Ausdruck verwenden:
//		cbCount.addActionListener(e ->  {
//				numClicks++;
//				lblText.setText(lblPrefix + numClicks);
//				if (iconInd < icons.length-1)
//					iconInd++;
//				else
//					iconInd = 0;
//				lblIcon.setIcon(icons[iconInd]);
//		  });
		
		// Der JFrame erhält einen Event Handler für Fenster-Ereignisse.
		frame.addWindowListener(new WiLi());
		
		// Diese Methode verhindert ein Schließen unter Missachtung des WindowListeners
		frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		
		// initiale JFrame-Größe festlegen
		frame.setSize(340, 150);
		
		// JFrame anzeigen
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		new MPC();
	}

	// Der WindowListener für den JFrame (= Top Level Container) wird als innere Klasse def.
	private class WiLi extends WindowAdapter {
		@Override
		public void windowClosing(WindowEvent e) {
			if (JOptionPane.showConfirmDialog(e.getWindow(), 
				  "Wollen Sie nach "+ numClicks +
				  " Klicks wirklich schon aufhören?", 
				  TITLE, JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
				System.exit(0);
		}
	}
}
